#!/usr/bin/python
# -*- coding: utf-8 -*-

#  StartupToIdle
#
#  Coded by Sven H (c) 2022
#  based on StartupToStandby
#  Support: https://dreambox.de/board/
#
# This plugin is open source but it is NOT free software.
#
# This plugin may only be distributed to and executed on hardware which
# is licensed by Dream Property GmbH.
# In other words:
# It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
# to hardware which is NOT licensed by Dream Property GmbH.
# It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
# on hardware which is NOT licensed by Dream Property GmbH.
#
# If you want to use or modify the code or parts of it, you have to keep the license.

from __future__ import print_function
from Plugins.Plugin import PluginDescriptor
from Components.config import config, ConfigSubsection, ConfigSelection, ConfigYesNo
from StartupToIdleConfiguration import StartupToIdleConfiguration, version_txt
from Screens.Standby import Standby, inStandby
from Screens.MessageBox import MessageBox
from Tools.DreamboxHardware import getFPWakeuptime, getFPWasTimerWakeup
from Tools import Notifications
from os import path as os_path, remove as os_remove
from time import time, localtime, strftime
from enigma import getExitCode
from . import _

#config
config.plugins.startuptoidle = ConfigSubsection()
startuptoidle_option = []
startuptoidle_option.append(("no", _("no") + " (%s)" % _("default")))
startuptoidle_option.append(("always", _("always")))
startuptoidle_option.append(("crashOnIdle", _("only on restart after 'power cut' or 'crash on idle mode'")))
config.plugins.startuptoidle.enabled = ConfigSelection(choices = startuptoidle_option, default = "no")
startuptoidle_manual_restart_option = []
startuptoidle_manual_restart_option.append(("last", _("last mode")))
startuptoidle_manual_restart_option.append(("idle", _("idle mode")))
startuptoidle_manual_restart_option.append(("active", _("active mode") + " (%s)" % _("default")))
config.plugins.startuptoidle.onManualRestart = ConfigSelection(choices = startuptoidle_manual_restart_option, default = "active")
config.plugins.startuptoidle.showdebuginfo = ConfigYesNo(default = False)
config.plugins.startuptoidle.savestartlog = ConfigYesNo(default = False)

#globals
shutdownActions = {
		1: "standby", 
		2: "reboot", 
		3: "restart", 
		8: "standby", 
		143: "restart/reboot console"
	}
lastState = None
lastStartLog = ""
startupToIdleLastStateFile = "/etc/enigma2/.StartupToIdleLastState"
startupToIdleTmpCheckFile = "/tmp/.StartupToIdleCheck"

def getBoxModel():
	#dm529,dm525,dm820,dm7080,dm900,dm920,one,two
	boxmodel="unknown"
	if os_path.exists("/proc/stb/info/model"):
		with open("/proc/stb/info/model", "r") as f:
			boxmodel = f.read().replace("\n","").replace("\l","")
	return boxmodel

boxModel = getBoxModel()

def sti_print(print_txt=""):
	if config.plugins.startuptoidle.showdebuginfo.value:
		print(print_txt)
	
def getKernelBootTime():
	if os_path.exists("/proc/stb/fp/rtc"):
		#one,two,dm820,dm7080
		kernel_boot_time = int(open("/proc/stb/fp/rtc","r").read())
		boot_time_target = "from rtc-file"
	else:
		#dm9x0, dm5x0 ??
		kernel_boot_time = os_path.getctime("/dev")
		boot_time_target = "from dev-ctime"
	return kernel_boot_time, boot_time_target

def writeCurrentState(state_txt="", add_log_txt="", usePrint=True):
	global lastState
	lastState = state_txt
	with open(startupToIdleLastStateFile, "w") as f:
		f.write(state_txt)
	if usePrint:
		sti_print("[StartupToIdle] %s - writeCurrentState: '%s'" % (add_log_txt, state_txt))

def onIdle(configElement):
	from Screens.Standby import inStandby
	inStandby.onClose.append(onLeaveIdle)
	writeCurrentState("idle", "onIdle")

def onLeaveIdle():
	if getExitCode() == 0:
		writeCurrentState("active", "onLeaveIdle")

def getLastState():
	lastState=""
	if os_path.exists(startupToIdleLastStateFile):
		lastState = str(open(startupToIdleLastStateFile,"r").read())
	return lastState

def autostart(reason, **kwargs):
	if reason == 1:
		shutdownAction = getExitCode()
		sti_print("[StartupToIdle] shutdown - shutdownAction: '%s' (%s)" % (shutdownActions.get(shutdownAction),shutdownAction))
		if shutdownAction in (1, 8): #on going to standby
			writeCurrentState("standby", "shutdown")
		elif shutdownAction == 2: #on reboot
			writeCurrentState(lastState + " " + shutdownActions.get(shutdownAction), "shutdown")
		elif shutdownAction in (3, 143): #on restart/restart service
			writeCurrentState(lastState + " " + shutdownActions.get(shutdownAction), "shutdown")
		else:
			writeCurrentState(lastState + " unknown shutdownaction (" + str(shutdownAction) + ")", "shutdown")

def sessionstart(reason, **kwargs):
	global lastStartLog
	output_txt = []
	if kwargs.has_key("session") and reason == 0:
		config.misc.standbyCounter.addNotifier(onIdle, initial_call=False)
		session = kwargs["session"]
		output_txt.append("[StartupToIdle] sessionstart - set onIdle-function")
	else:
		output_txt.append("[StartupToIdle] sessionstart - can't set onIdle-function")
		session = None
	
	lastStateValue = getLastState()
	isReboot = not os_path.exists(startupToIdleTmpCheckFile)
	isCrash = lastStateValue in ("active", "idle") and not isReboot
	isManualRestart = "restart" in lastStateValue or "reboot" in lastStateValue
	enabledAlways = config.plugins.startuptoidle.enabled.value == "always"
	enabledOnCrash = config.plugins.startuptoidle.enabled.value == "crashOnIdle" and isCrash and lastStateValue == "idle"
	
	#check for powercut
	kernel_boot_time, boot_time_target = getKernelBootTime()
	wakeup_time = getFPWakeuptime()
	was_timer_wakeup = False
	if session:
		was_timer_wakeup = session.nav.wasTimerWakeup()
	isPowercut = kernel_boot_time < 1000 and wakeup_time == 0 and was_timer_wakeup == False and isReboot and not isManualRestart
	enabledAfterPowerCut = config.plugins.startuptoidle.enabled.value == "crashOnIdle" and isPowercut and lastStateValue!="active"
	
	#debug output
	output_txt.append("[StartupToIdle] sessionstart - plugin version: %s (box: %s)" % (version_txt,boxModel))
	output_txt.append("[StartupToIdle] sessionstart - setup-option 'startup to idle mode': %s" % (config.plugins.startuptoidle.enabled.value))
	output_txt.append("[StartupToIdle] sessionstart - setup-option 'after manual Restart GUI/Restart going to': %s" % (config.plugins.startuptoidle.onManualRestart.value))
	output_txt.append("[StartupToIdle] sessionstart - system_time: %s = %s" % (time(), strftime("%d.%m.%Y %H:%M:%S", localtime(float(time())))))
	output_txt.append("[StartupToIdle] sessionstart - kernel_boot_time: %s = %s (%s)" % (kernel_boot_time, strftime("%d.%m.%Y %H:%M:%S", localtime(float(kernel_boot_time))), boot_time_target))
	output_txt.append("[StartupToIdle] sessionstart - wakeuptime: %s = %s" % (wakeup_time, strftime("%d.%m.%Y %H:%M:%S", localtime(float(wakeup_time)))))
	output_txt.append("[StartupToIdle] sessionstart - was_timer_wakeup: %s" % (was_timer_wakeup))
	output_txt.append("[StartupToIdle] sessionstart - isReboot: %s" % (isReboot))
	output_txt.append("[StartupToIdle] sessionstart - isManualRestart: %s" % (isManualRestart))
	output_txt.append("[StartupToIdle] sessionstart - isCrash: %s" % (isCrash))
	output_txt.append("[StartupToIdle] sessionstart - isPowercut: %s" % (isPowercut))
	output_txt.append("[StartupToIdle] sessionstart - lastState: %s" % (lastStateValue))
	output_txt.append("[StartupToIdle] sessionstart - enable idle => always: %s, onCrash: %s, afterPowerCut: %s" % (enabledAlways, enabledOnCrash, enabledAfterPowerCut))
	
	if reason == 0 and not inStandby :
		if (enabledAlways or enabledOnCrash or enabledAfterPowerCut):
			output_txt.append("[StartupToIdle] sessionstart - going to idle")
			Notifications.AddNotificationWithID("Standby", Standby)
			
			#show info message
			if enabledAfterPowerCut:
				output_txt.append("[StartupToIdle] sessionstart - show message")
				message = _("StartupToIdle\n\nThe box was set to idle mode on restart after a power cut.")
				Notifications.AddPopup(message, MessageBox.TYPE_INFO, 30)
			elif enabledOnCrash:
				output_txt.append("[StartupToIdle] sessionstart - show message")
				message = _("StartupToIdle\n\nThe box was set to idle mode after a green screen (crash) on last idle mode.\n\nA possible crashlog-file you can find in '/media/hdd/'.")
				Notifications.AddPopup(message, MessageBox.TYPE_INFO, 30)
		else:
			onManualRestart = config.plugins.startuptoidle.onManualRestart.value
			startToIdle = (lastStateValue.startswith("idle") and  onManualRestart != "active") or onManualRestart == "idle"
			if startToIdle:
				output_txt.append("[StartupToIdle] sessionstart - going to idle mode")
				Notifications.AddNotificationWithID("Standby", Standby)
			else:
				output_txt.append("[StartupToIdle] sessionstart - going to active mode")
				writeCurrentState("active", "sessionstart", usePrint=False)
				output_txt.append("[StartupToIdle] sessionstart - writeCurrentState: 'active'")
	else:
		output_txt.append("[StartupToIdle] sessionstart - nothing todo - already in idle mode")
	
	#output debug data
	for entry in output_txt:
		sti_print(entry)
	lastStartLog = "\n".join(output_txt).replace("[StartupToIdle] sessionstart - ","")
	if config.plugins.startuptoidle.savestartlog.value:
		with open("/tmp/StartupToIdle_lastPluginStart.log","w") as f:
			f.write(lastStartLog)
	
	sti_print("[StartupToIdle] sessionstart - create idleTmpCheck-file")
	open(startupToIdleTmpCheckFile,"a").close()

def main(session, **kwargs):
	sti_print("[StartupToIdle] Open Config Screen")
	session.open(StartupToIdleConfiguration)

def Plugins(path, **kwargs):
	pname = "StartupToIdle"
	pdesc = _("Startup To Idle Mode") + " (v%s)" % version_txt
	return [
	PluginDescriptor(name=pname, description = pdesc, where = PluginDescriptor.WHERE_PLUGINMENU, fnc = main, needsRestart = False, icon = "plugin.png"),
	PluginDescriptor(name=pname, description = pdesc, where = PluginDescriptor.WHERE_SESSIONSTART, fnc = sessionstart, needsRestart = False, weight = -1),
	PluginDescriptor(name=pname, where = PluginDescriptor.WHERE_AUTOSTART, fnc = autostart)
	]

